@@ -14,6 +14,7 @@
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
+#define PY_SSIZE_T_CLEAN
 #include "Python.h"
 #include "structmember.h"
 #include "_radix/radix.h"
@@ -50,8 +51,6 @@ PyObject *radix_constructor;
 typedef struct {
         PyObject_HEAD
         PyObject *user_attr;    /* User-specified attributes */
-        PyObject *network;
-        PyObject *prefix;
         PyObject *prefixlen;
         PyObject *family;
         PyObject *packed;
@@ -64,10 +63,9 @@ static RadixNodeObject *
 newRadixNodeObject(radix_node_t *rn)
 {
         RadixNodeObject *self;
-        char network[256], prefix[256];
 
         /* Sanity check */
-        if (rn == NULL || rn->prefix == NULL || 
+        if (rn == NULL || rn->prefix == NULL ||
             (rn->prefix->family != AF_INET && rn->prefix->family != AF_INET6))
                 return NULL;
 
@@ -77,24 +75,16 @@ newRadixNodeObject(radix_node_t *rn)
 
         self->rn = rn;
 
-        /* Format addresses for packing into objects */
-        prefix_addr_ntop(rn->prefix, network, sizeof(network));
-        prefix_ntop(rn->prefix, prefix, sizeof(prefix));
-
-        self->user_attr = PyDict_New();
-        self->network = PyString_FromString(network);
-        self->prefix = PyString_FromString(prefix);
+        self->user_attr = NULL;
         self->prefixlen = PyInt_FromLong(rn->prefix->bitlen);
         self->family = PyInt_FromLong(rn->prefix->family);
         self->packed = PyString_FromStringAndSize((char*)&rn->prefix->add,
             rn->prefix->family == AF_INET ? 4 : 16);
 
-        if (self->user_attr == NULL || self->prefixlen == NULL || 
-            self->family == NULL || self->network == NULL || 
-            self->prefix == NULL) {
-                /* RadixNode_dealloc will clean up for us */
-                Py_XDECREF(self);
-                return (NULL);          
+        if (self->family == NULL || self->prefixlen == NULL) {
+            /* RadixNode_dealloc will clean up for us */
+            Py_XDECREF(self);
+            return (NULL);
         }
 
         return self;
@@ -108,8 +98,6 @@ RadixNode_dealloc(RadixNodeObject *self)
         Py_XDECREF(self->user_attr);
         Py_XDECREF(self->prefixlen);
         Py_XDECREF(self->family);
-        Py_XDECREF(self->network);
-        Py_XDECREF(self->prefix);
         Py_XDECREF(self->packed);
         PyObject_Del(self);
 }
@@ -138,10 +126,24 @@ Radix_parent(RadixNodeObject *self, void *closure)
         }
         Py_RETURN_NONE;
 }
+static PyObject *
+Radix_prefix(RadixNodeObject *self, void *closure)
+{
+        char buf[256];
+        return PyString_FromString(
+                prefix_ntop(self->rn->prefix, buf, sizeof(buf))
+        );
+}
+static PyObject *
+Radix_network(RadixNodeObject *self, void *closure)
+{
+        char buf[256];
+        return PyString_FromString(
+                prefix_addr_ntop(self->rn->prefix, buf, sizeof(buf))
+        );
+}
 static PyMemberDef RadixNode_members[] = {
-        {"data",        T_OBJECT, offsetof(RadixNodeObject, user_attr), READONLY},
-        {"network",     T_OBJECT, offsetof(RadixNodeObject, network),   READONLY},
-        {"prefix",      T_OBJECT, offsetof(RadixNodeObject, prefix),    READONLY},
+        {"data",        T_OBJECT, offsetof(RadixNodeObject, user_attr), 0},
         {"prefixlen",   T_OBJECT, offsetof(RadixNodeObject, prefixlen), READONLY},
         {"family",      T_OBJECT, offsetof(RadixNodeObject, family),    READONLY},
         {"packed",      T_OBJECT, offsetof(RadixNodeObject, packed),    READONLY},
@@ -155,10 +157,22 @@ static PyGetSetDef node_getter[] = {
          "parent of node",           /* optional doc string */
          NULL                        /* optional additional data for getter and setter */
         },
+        {"prefix",
+         (getter) Radix_prefix,      /* C function to get the attribute */
+         NULL,                       /* C function to set the attribute */
+         "Node prefix",              /* optional doc string */
+         NULL                        /* optional additional data for getter and setter */
+        },
+        {"network",
+         (getter) Radix_network,      /* C function to get the attribute */
+         NULL,                       /* C function to set the attribute */
+         "Node network",              /* optional doc string */
+         NULL                        /* optional additional data for getter and setter */
+        },
         {NULL}  /* Sentinel */
 };
 
-PyDoc_STRVAR(RadixNode_doc, 
+PyDoc_STRVAR(RadixNode_doc,
 "Node in a radix tree");
 
 static PyTypeObject RadixNode_Type = {
@@ -256,7 +270,7 @@ Radix_dealloc(RadixObject *self)
 }
 
 static prefix_t
-*args_to_prefix(prefix_t *prefix, char *addr, char *packed, int packlen, long prefixlen)
+*args_to_prefix(prefix_t *prefix, char *addr, char *packed, Py_ssize_t packlen, long prefixlen)
 {
         prefix_t *old_prefix = prefix;
         const char *errmsg;
@@ -279,7 +293,7 @@ static prefix_t
                             "Invalid address format");
                 }
         } else if (packed != NULL) {    /* "parse" a packed binary address */
-                if ((prefix = prefix_from_blob_ex(prefix, (u_char*)packed, packlen, 
+                if ((prefix = prefix_from_blob_ex(prefix, (u_char*)packed, packlen,
                     prefixlen)) == NULL) {
                         PyErr_SetString(PyExc_ValueError,
                             "Invalid packed address format");
@@ -308,7 +322,7 @@ create_add_node(RadixObject *self, prefix_t *prefix)
 
         /*
          * Create a RadixNode object in the data area of the node
-         * We duplicate most of the node's identity, because the radix.c:node 
+         * We duplicate most of the node's identity, because the radix.c:node
          * itself has a lifetime independent of the Python node object
          * Confusing? yeah...
          */
@@ -352,7 +366,7 @@ Radix_add(RadixObject *self, PyObject *args, PyObject *kw_args)
 
         char *addr = NULL, *packed = NULL;
         long prefixlen = -1;
-        int packlen = -1;
+        Py_ssize_t packlen = -1;
 
         if (!PyArg_ParseTupleAndKeywords(args, kw_args, "|zlz#:add", keywords,
             &addr, &prefixlen, &packed, &packlen))
@@ -381,7 +395,7 @@ Radix_delete(RadixObject *self, PyObject *args, PyObject *kw_args)
 
         char *addr = NULL, *packed = NULL;
         long prefixlen = -1;
-        int packlen = -1;
+        Py_ssize_t packlen = -1;
 
         if (!PyArg_ParseTupleAndKeywords(args, kw_args, "|zlz#:delete", keywords,
             &addr, &prefixlen, &packed, &packlen))
@@ -424,7 +438,7 @@ Radix_search_exact(RadixObject *self, PyObject *args, PyObject *kw_args)
 
         char *addr = NULL, *packed = NULL;
         long prefixlen = -1;
-        int packlen = -1;
+        Py_ssize_t packlen = -1;
 
         if (!PyArg_ParseTupleAndKeywords(args, kw_args, "|zlz#:search_exact", keywords,
             &addr, &prefixlen, &packed, &packlen))
@@ -462,7 +476,7 @@ Radix_search_best(RadixObject *self, PyObject *args, PyObject *kw_args)
 
         char *addr = NULL, *packed = NULL;
         long prefixlen = -1;
-        int packlen = -1;
+        Py_ssize_t packlen = -1;
 
         if (!PyArg_ParseTupleAndKeywords(args, kw_args, "|zlz#:search_best", keywords,
             &addr, &prefixlen, &packed, &packlen))
@@ -470,7 +484,7 @@ Radix_search_best(RadixObject *self, PyObject *args, PyObject *kw_args)
         if ((prefix = args_to_prefix(&lprefix, addr, packed, packlen, prefixlen)) == NULL)
                 return NULL;
 
-        if ((node = radix_search_best(self->rt, prefix)) == NULL || 
+        if ((node = radix_search_best(self->rt, prefix)) == NULL ||
             node->data == NULL) {
                 Py_INCREF(Py_None);
                 return Py_None;
@@ -500,7 +514,7 @@ Radix_search_worst(RadixObject *self, PyObject *args, PyObject *kw_args)
 
         char *addr = NULL, *packed = NULL;
         long prefixlen = -1;
-        int packlen = -1;
+        Py_ssize_t packlen = -1;
 
         if (!PyArg_ParseTupleAndKeywords(args, kw_args, "|zlz#:search_worst", keywords,
             &addr, &prefixlen, &packed, &packlen))
@@ -508,7 +522,7 @@ Radix_search_worst(RadixObject *self, PyObject *args, PyObject *kw_args)
         if ((prefix = args_to_prefix(&lprefix, addr, packed, packlen, prefixlen)) == NULL)
                 return NULL;
 
-        if ((node = radix_search_worst(self->rt, prefix)) == NULL || 
+        if ((node = radix_search_worst(self->rt, prefix)) == NULL ||
             node->data == NULL) {
                 Py_INCREF(Py_None);
                 return Py_None;
@@ -524,7 +538,7 @@ add_node_to_list(radix_node_t *node, void *arg)
         PyObject *ret = arg;
 
         if (node->data != NULL)
-                PyList_Append(ret, ((RadixNodeObject *)node->data));
+                PyList_Append(ret, (PyObject *)((RadixNodeObject *)node->data));
         return (0);
 }
 
@@ -543,7 +557,7 @@ Radix_search_covered(RadixObject *self, PyObject *args, PyObject *kw_args)
 
         char *addr = NULL, *packed = NULL;
         long prefixlen = -1;
-        int packlen = -1;
+        Py_ssize_t packlen = -1;
 
         if (!PyArg_ParseTupleAndKeywords(args, kw_args, "|zlz#:search_covered", keywords, &addr, &prefixlen, &packed, &packlen))
                 return NULL;
@@ -576,7 +590,7 @@ Radix_search_covering(RadixObject *self, PyObject *args, PyObject *kw_args)
 
         char *addr = NULL, *packed = NULL;
         long prefixlen = -1;
-        int packlen = -1;
+        Py_ssize_t packlen = -1;
 
         if (!PyArg_ParseTupleAndKeywords(args, kw_args, "|zlz#:search_covering", keywords, &addr, &prefixlen, &packed, &packlen)) {
                 return NULL;
@@ -643,8 +657,14 @@ Radix_prefixes(RadixObject *self, PyObject *args)
 
         RADIX_TREE_WALK(self->rt, node) {
                 if (node->data != NULL) {
-                        PyList_Append(ret,
-                            ((RadixNodeObject *)node->data)->prefix);
+                        char buf[256];
+                        PyObject *prefix = PyString_FromString(
+                                prefix_ntop(node->prefix, buf, sizeof(buf))
+                        );
+                        if (prefix != NULL) {
+                            PyList_Append(ret, prefix);
+                            Py_XDECREF(prefix);
+                        }
                 }
         } RADIX_TREE_WALK_END;
 
@@ -806,7 +826,7 @@ RadixIter_iternext(RadixIterObject *self)
         return (ret);
 }
 
-PyDoc_STRVAR(RadixIter_doc, 
+PyDoc_STRVAR(RadixIter_doc,
 "Radix tree iterator");
 
 static PyTypeObject RadixIter_Type = {
